Rem
Rem $Header: catcr.sql 03-mar-2005.12:39:30 rburns Exp $
Rem
Rem catcr.sql
Rem
Rem Copyright (c) 2001, 2005, Oracle. All rights reserved.  
Rem
Rem    NAME
Rem      catcr.sql - CATalog Component Registry
Rem
Rem    DESCRIPTION
Rem      This script is a trimmed down version of the 9.2 catcr.sql script
Rem      to use for Critical Patch Updates in 9.0.1.4, 9.0.1.5,  and 8.1.7.4
Rem
Rem    NOTES
Rem      Use SQLPLUS or svrmgrl
Rem      Conned AS SYSDBA
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem    rburns      03/03/05  Created


Rem -------------------------------------------------------------------------
Rem DBMS REGISTRY PACKAGE
Rem -------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE dbms_registry AS

nothing_script        CONSTANT VARCHAR2(35) := '@nothing.sql';


FUNCTION  script      (comp_id IN VARCHAR2, 
                       script_name IN VARCHAR2) RETURN VARCHAR2;

END;
/

CREATE OR REPLACE PACKAGE BODY dbms_registry 
AS



------------------------- option_OK ---------------------------------

FUNCTION option_OK (option_name VARCHAR2) RETURN BOOLEAN
IS

option_value v$option.value%type;
OK           BOOLEAN := FALSE;

BEGIN
   -- Get option name 

   IF option_name IS NULL THEN
      OK := TRUE;   -- No option required
   ELSE
      SELECT value INTO option_value FROM v$option
      WHERE parameter = option_name;
      IF option_value = 'TRUE' THEN
         OK := TRUE;
      END IF;
   END IF;      

   RETURN OK; 

END option_OK;


------------------------- disable DDL triggers -------------------------

PROCEDURE disable_ddl_triggers (schema varchar2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;  -- executes DDL 

p_schema       VARCHAR2(30) := NLS_UPPER(schema);
p_trigger_name VARCHAR2(30);
TYPE CurTyp    IS REF CURSOR;  
trigger_cv     CurTyp;  

BEGIN

  IF p_schema IN ('SYS','SYSTEM') THEN
     RETURN;  -- don't disable any triggers
  END IF;
  -- Use dynamic SQL to avoid errors during DB creation
  OPEN trigger_cv FOR
         'SELECT trigger_name FROM dba_triggers
          WHERE owner = :p_schema AND
                status = ''ENABLED'' AND
                trigger_type IN (''BEFORE EVENT'', ''AFTER EVENT'')'
        USING p_schema;

  LOOP
      FETCH trigger_cv INTO p_trigger_name;  
      EXIT WHEN trigger_cv%NOTFOUND; 
      EXECUTE IMMEDIATE 'ALTER TRIGGER ' || 
                         p_schema || '.' || p_trigger_name ||
                         ' DISABLE';
  END LOOP;

END disable_ddl_triggers;


-------------------------- SCRIPT --------------------------------

FUNCTION script(comp_id IN VARCHAR2, 
                script_name IN VARCHAR2) RETURN VARCHAR2 
IS

p_cid    VARCHAR2(30) := NLS_UPPER(comp_id);
p_schema VARCHAR2(30);
p_null   CHAR(1);
found    BOOLEAN := FALSE;
BEGIN

   -- JAVA
   IF p_cid = 'JAVAVM' THEN
      BEGIN
         SELECT NULL into p_null FROM obj$ WHERE type#=29 AND
              owner#=0 and rownum <=1;
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- XML
   ELSIF p_cid = 'XML' THEN
      BEGIN
        SELECT NULL INTO p_null FROM obj$ WHERE type#=29
         AND owner#=0  AND name like 'oracle/xml/parser%' and ROWNUM<=1;
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- CONTEXT (CTX)
   ELSIF p_cid = 'CONTEXT' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'CTXSYS';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Real Application Clusters
   ELSIF p_cid = 'RAC' THEN
      BEGIN
         SELECT null INTO p_null FROM obj$
             WHERE owner#=0 AND name='V$PING' AND ROWNUM <=1;
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Oracle Workspace Manager
   ELSIF p_cid = 'OWM' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'WMSYS';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN  -- look for OWM table in SYSTEM
           BEGIN
             SELECT NULL INTO p_null FROM obj$ o, user$ u
             WHERE u.name = 'SYSTEM' AND
                 o.name = 'WM$ENV_VARS' AND
                 u.user#=o.owner# and o.type#=2;
             found := TRUE;
           EXCEPTION
              WHEN NO_DATA_FOUND THEN NULL;
           END;
      END;

   -- Spatial
   ELSIF p_cid = 'SDO' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'MDSYS';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Intermedia
   ELSIF p_cid = 'ORDIM' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'ORDSYS';
         EXECUTE IMMEDIATE 'SELECT NULL ' ||
                           'FROM ordsys.ord_installations ' ||
                           'WHERE short_name = ''ORDIM'''
                            INTO p_null;
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Oracle Label Security
   ELSIF p_cid = 'OLS' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'LBACSYS';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Oracle Data Mining
   ELSIF p_cid = 'ODM' THEN
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'ODM';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Ultrasearch
   ELSIF p_cid = 'WK' THEN 
      BEGIN
         SELECT NULL into p_null FROM user$ WHERE name = 'WKSYS';
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- CWMLite (OLAP Catalog)
   ELSIF p_cid = 'AMD' THEN
      BEGIN
        SELECT NULL into p_null FROM user$ WHERE name = 'OLAPSYS';
        found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   -- Replication (817 catcr.sql only)
   ELSIF p_cid = 'REP' THEN
      BEGIN
        SELECT NULL INTO p_null FROM obj$ WHERE type#=2 
         AND name like 'REPCAT$%' and ROWNUM<=1;
         found := TRUE;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN NULL;
      END;

   END IF;

   IF found = FALSE THEN
      RETURN dbms_registry.nothing_script;
   END IF;

  -- Component found, check options

  -- look for components that require options (SE database and OLS)
  IF (p_cid = 'OLS' AND NOT option_OK('Oracle Label Security')) OR
     (p_cid IN ('APS','XOQ','AMD') AND NOT option_OK('OLAP')) OR
     (p_cid = 'ODM' AND NOT option_OK('Oracle Data Mining')) THEN
       IF p_cid = 'OLS' THEN    -- login trigger causes errors
          disable_ddl_triggers('LBACSYS');
       END IF;
       RETURN dbms_registry.nothing_script;
  ELSE
     RETURN (script_name);
  END IF;

END script;


END dbms_registry;
/

show errors

CREATE OR REPLACE PUBLIC SYNONYM dbms_registry FOR dbms_registry;


